using System.Windows;
using Infragistics.Controls.Editors;
using System.Linq;

namespace BindSelected
{
    public class XamComboEditorEx : XamComboEditor
    {
        public XamComboEditorEx()
            : base()
        {
            this.SelectionChanged += new System.EventHandler(XamComboEditorEx_SelectionChanged);
        }

        #region SelectedValuePath
        public string SelectedValuePath
        {
            get { return (string)GetValue(SelectedValuePathProperty); }
            set { SetValue(SelectedValuePathProperty, value); }
        }

        public static readonly DependencyProperty SelectedValuePathProperty =
            DependencyProperty.Register("SelectedValuePath", typeof(string), typeof(XamComboEditorEx),
            new PropertyMetadata(new PropertyChangedCallback(SelectedValuePath_Changed)));

        private static void SelectedValuePath_Changed(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var combo = d as XamComboEditorEx;
            if (combo != null)
            {
                combo.ResetSelectedValue();
            }
        }
        #endregion //SelectedValuePath

        #region SelectedValue
        public object SelectedValue
        {
            get { return (object)GetValue(SelectedValueProperty); }
            set
            {
                var q = (from item in Items
                         where item.Data.GetType().GetProperty(SelectedValuePath).GetValue(item.Data, null).Equals(value)
                         select item).FirstOrDefault(); // You could probably use First instead of Single if you expect to
                // have multiple items with the same SelectedValue value
                if (q == null)
                    this.SelectedItem = null;
                else
                    this.SelectedItem = q.Data;
            }
        }

        public static readonly DependencyProperty SelectedValueProperty =
            DependencyProperty.Register("SelectedValue", typeof(object), typeof(XamComboEditorEx), null);
        #endregion //SelectedValue

        void XamComboEditorEx_SelectionChanged(object sender, System.EventArgs e)
        {
            var combo = sender as XamComboEditorEx;
            if (combo != null)
                combo.ResetSelectedValue();
        }

        private void ResetSelectedValue()
        {
            if (this.SelectedItem != null && this.SelectedItem.GetType().GetProperty(this.SelectedValuePath) != null)
            {
                var propInfo = this.SelectedItem.GetType().GetProperty(this.SelectedValuePath);
                this.SetSelectedValueSilently(propInfo.GetValue(this.SelectedItem, null));
            }
            else
                this.SetSelectedValueSilently(null);

        }

        private void SetSelectedValueSilently(object value)
        {
            this.SetValue(XamComboEditorEx.SelectedValueProperty, value);
        }
    }
}

